import { MessagePublisherPort } from '@mobicoop/ddd-library';
import {
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
  AD_ROUTE_PROVIDER,
} from '@modules/ad/ad.di-tokens';
import { AdFactory } from '@modules/ad/core/domain/ad.factory';
import { Inject } from '@nestjs/common';
import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { MATCHER_AD_UPDATE_FAILED_ROUTING_KEY } from '@src/app.constants';
import { GeorouterService } from '../../../domain/georouter.service';
import { MatcherAdUpdateFailedIntegrationEvent } from '../../events/matcher-ad-failure.integration-event';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { UpdateAdCommand } from './update-ad.command';

@CommandHandler(UpdateAdCommand)
export class UpdateAdService implements ICommandHandler {
  constructor(
    @Inject(AD_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(AD_ROUTE_PROVIDER)
    private readonly routeProvider: GeorouterService,
  ) {}

  async execute(command: UpdateAdCommand): Promise<void> {
    try {
      const adFactory = new AdFactory(this.routeProvider);
      const ad = await adFactory.create(command);
      return this.repository.update(ad.id, ad);
    } catch (error: any) {
      const integrationEvent = new MatcherAdUpdateFailedIntegrationEvent({
        id: command.id,
        metadata: {
          correlationId: command.id,
          timestamp: Date.now(),
        },
        cause: error.message,
      });
      this.messagePublisher.publish(
        MATCHER_AD_UPDATE_FAILED_ROUTING_KEY,
        JSON.stringify(integrationEvent),
      );
      throw error;
    }
  }
}
