import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { MatcherAdCreatedDomainEvent } from '../../domain/events/matcher-ad-created.domain-event';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { AD_MESSAGE_PUBLISHER } from '@modules/ad/ad.di-tokens';
import { MATCHER_AD_CREATED_ROUTING_KEY } from '@src/app.constants';
import { MatcherAdCreatedIntegrationEvent } from '../events/matcher-ad-created.integration-event';

@Injectable()
export class PublishMessageWhenMatcherAdIsCreatedDomainEventHandler {
  constructor(
    @Inject(AD_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(MatcherAdCreatedDomainEvent.name, { async: true, promisify: true })
  async handle(event: MatcherAdCreatedDomainEvent): Promise<any> {
    const matcherAdCreatedIntegrationEvent =
      new MatcherAdCreatedIntegrationEvent({
        id: event.aggregateId,
        driverDuration: event.driverDuration,
        driverDistance: event.driverDistance,
        passengerDuration: event.passengerDuration,
        passengerDistance: event.passengerDistance,
        fwdAzimuth: event.fwdAzimuth,
        backAzimuth: event.backAzimuth,
        metadata: event.metadata,
      });
    this.messagePublisher.publish(
      MATCHER_AD_CREATED_ROUTING_KEY,
      JSON.stringify(matcherAdCreatedIntegrationEvent),
    );
  }
}
