import { PointProps } from './value-objects/point.value-object';
import { ScheduleItemProps } from './value-objects/schedule-item.value-object';

/**
 * The data provided by the end-user to publish an ad
 */
export interface UserAd {
  id: string;
  driver: boolean;
  passenger: boolean;
  frequency: Frequency;
  fromDate: string;
  toDate: string;
  schedule: ScheduleItemProps[];
  seatsProposed: number;
  seatsRequested: number;
  strict: boolean;
  waypoints: PointProps[];
}

// All properties that an Ad has
export interface AdProps {
  driver: boolean;
  passenger: boolean;
  frequency: Frequency;
  fromDate: string;
  toDate: string;
  schedule: ScheduleItemProps[];
  seatsProposed: number;
  seatsRequested: number;
  strict: boolean;
  driverDuration?: number;
  driverDistance?: number;
  passengerDuration?: number;
  passengerDistance?: number;
  waypoints: PointProps[];
  points: PointProps[];
  fwdAzimuth: number;
  backAzimuth: number;
}

// Properties that are needed for an Ad creation
export interface CreateAdProps {
  id: string;
  driver: boolean;
  passenger: boolean;
  frequency: Frequency;
  fromDate: string;
  toDate: string;
  schedule: ScheduleItemProps[];
  seatsProposed: number;
  seatsRequested: number;
  strict: boolean;
  waypoints: PointProps[];
  driverDuration?: number;
  driverDistance?: number;
  passengerDuration?: number;
  passengerDistance?: number;
  fwdAzimuth: number;
  backAzimuth: number;
  points: PointProps[];
}

export enum Frequency {
  PUNCTUAL = 'PUNCTUAL',
  RECURRENT = 'RECURRENT',
}

export enum Role {
  DRIVER = 'DRIVER',
  PASSENGER = 'PASSENGER',
}
