import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { ActorTime, ActorTimeProps } from './actor-time.value-object';
import { Step, StepProps } from './step.value-object';
import { Role } from '../ad.types';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface JourneyItemProps extends StepProps {
  actorTimes: ActorTimeProps[];
}

export class JourneyItem extends ValueObject<JourneyItemProps> {
  get duration(): number {
    return this.props.duration;
  }

  get distance(): number | undefined {
    return this.props.distance;
  }

  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  get actorTimes(): ActorTimeProps[] {
    return this.props.actorTimes;
  }

  driverTime = (): string => {
    const driverTime: Date = (
      this.actorTimes.find(
        (actorTime: ActorTime) => actorTime.role == Role.DRIVER,
      ) as ActorTime
    ).firstDatetime;
    return `${driverTime.getUTCHours().toString().padStart(2, '0')}:${driverTime
      .getUTCMinutes()
      .toString()
      .padStart(2, '0')}`;
  };

  protected validate(props: JourneyItemProps): void {
    // validate step props
    new Step({
      lon: props.lon,
      lat: props.lat,
      distance: props.distance,
      duration: props.duration,
    });
    if (props.actorTimes.length == 0)
      throw new ArgumentOutOfRangeException(
        'at least one actorTime is required',
      );
  }
}
