import { Inject, Injectable, OnModuleInit } from '@nestjs/common';
import { ClientGrpc } from '@nestjs/microservices';
import { GRPC_GEOROUTER_SERVICE_NAME } from '@src/app.constants';
import { Observable, lastValueFrom } from 'rxjs';
import { GEOGRAPHY_PACKAGE } from '../ad.di-tokens';
import {
  GeorouterService,
  RouteRequest,
  RouteResponse,
} from '../core/domain/georouter.service';

interface GeorouterPort {
  getRoute(request: RouteRequest): Observable<RouteResponse>;
}

@Injectable()
export class Georouter implements GeorouterService, OnModuleInit {
  private georouterService: GeorouterPort;

  constructor(@Inject(GEOGRAPHY_PACKAGE) private readonly client: ClientGrpc) {}

  onModuleInit() {
    this.georouterService = this.client.getService<GeorouterPort>(
      GRPC_GEOROUTER_SERVICE_NAME,
    );
  }

  getRoute = async (request: RouteRequest): Promise<RouteResponse> => {
    try {
      return await lastValueFrom(this.georouterService.getRoute(request));
    } catch (error: any) {
      throw error;
    }
  };
}
