import { RabbitSubscribe } from '@mobicoop/message-broker-module';
import { UpdateAdCommand } from '@modules/ad/core/application/commands/update-ad/update-ad.command';
import { Injectable } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import {
  AD_UPDATED_MESSAGE_HANDLER,
  AD_UPDATED_ROUTING_KEY,
} from '@src/app.constants';
import { Ad } from './ad.types';

@Injectable()
export class AdUpdatedMessageHandler {
  constructor(private readonly commandBus: CommandBus) {}

  @RabbitSubscribe({
    name: AD_UPDATED_MESSAGE_HANDLER,
    routingKey: AD_UPDATED_ROUTING_KEY,
  })
  public async adUpdated(message: string) {
    try {
      const updatedAd: { data: Ad } = JSON.parse(message);
      await this.commandBus.execute(new UpdateAdCommand(updatedAd.data));
    } catch (error: any) {
      // do not throw error to acknowledge incoming message
      // error handling should be done in the command handler, if relevant
    }
  }
}
