import {
  AD_MESSAGE_PUBLISHER,
  AD_REPOSITORY,
  AD_ROUTE_PROVIDER,
} from '@modules/ad/ad.di-tokens';
import { UpdateAdCommand } from '@modules/ad/core/application/commands/update-ad/update-ad.command';
import { UpdateAdService } from '@modules/ad/core/application/commands/update-ad/update-ad.service';
import { GeorouterService } from '@modules/ad/core/domain/georouter.service';
import { Test, TestingModule } from '@nestjs/testing';
import { createAdProps } from './ad.fixtures';

const mockAdRepository = {
  update: jest.fn().mockImplementation((id) => {
    if (id === '42') {
      throw 'Bad id!';
    }
  }),
};

const mockRouteProvider: GeorouterService = {
  getRoute: jest.fn().mockImplementation(() => ({
    distance: 350101,
    duration: 14422,
    fwdAzimuth: 273,
    backAzimuth: 93,
    distanceAzimuth: 336544,
    points: [
      {
        lon: 6.1765102,
        lat: 48.689445,
      },
      {
        lon: 4.984578,
        lat: 48.725687,
      },
      {
        lon: 2.3522,
        lat: 48.8566,
      },
    ],
  })),
};

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('update-ad.service', () => {
  let updateAdService: UpdateAdService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        {
          provide: AD_ROUTE_PROVIDER,
          useValue: mockRouteProvider,
        },
        {
          provide: AD_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        UpdateAdService,
      ],
    }).compile();

    updateAdService = module.get<UpdateAdService>(UpdateAdService);
  });

  it('should be defined', () => {
    expect(updateAdService).toBeDefined();
  });

  describe('execute', () => {
    it('should call the repository update method', async () => {
      const updateAdCommand = new UpdateAdCommand(createAdProps());
      await updateAdService.execute(updateAdCommand);
      expect(mockAdRepository.update).toHaveBeenCalled();
    });

    it('should emit an event when an error occurs', async () => {
      const commandProps = createAdProps();
      commandProps.id = '42';
      const updateAdCommand = new UpdateAdCommand(commandProps);
      await expect(updateAdService.execute(updateAdCommand)).rejects.toBe(
        'Bad id!',
      );
      expect(mockMessagePublisher.publish).toHaveBeenCalled();
    });
  });
});
